@@ -39,6 +39,8 @@ from .util import AID, CAPABILITY, TRANSPORT
 
 SW_OK = 0x9000
 SW_APPLICATION_NOT_FOUND = 0x6a82
+SW_NO_INPUT_DATA = 0x6285
+SW_CONDITIONS_NOT_SATISFIED = 0x6985
 
 INS_SELECT = 0xa4
 INS_YK4_CAPABILITIES = 0x1d
