@@ -28,7 +28,7 @@
 
 from __future__ import absolute_import
 from enum import IntEnum, unique
-from .driver_ccid import APDUError, SW_OK
+from .driver_ccid import APDUError, SW_OK, SW_APPLICATION_NOT_FOUND
 from .util import AID, Tlv, parse_tlvs
 from cryptography import x509
 from cryptography.utils import int_to_bytes, int_from_bytes
@@ -137,7 +137,6 @@ class OBJ(IntEnum):
     FINGERPRINTS = 0x5fc103
     SECURITY = 0x5fc106
     FACIAL = 0x5fc108
-    PRINTED = 0x5fc109
     SIGNATURE = 0x5fc10a  # cert for 9c key
     KEY_MANAGEMENT = 0x5fc10b  # cert for 9d key
     CARD_AUTH = 0x5fc101  # cert for 9e key
@@ -167,6 +166,7 @@ class OBJ(IntEnum):
     RETIRED20 = 0x5fc120
 
     PIVMAN_DATA = 0x5fff00
+    PIVMAN_PROTECTED_DATA = 0x5fc109  # Use slot for printed information.
     ATTESTATION = 0x5fff01
 
     @classmethod
@@ -365,6 +365,14 @@ class PivmanData(object):
     def puk_blocked(self, value):
         self._set_flag(0x01, value)
 
+    @property
+    def mgm_key_protected(self):
+        return self._get_flag(0x02)
+
+    @mgm_key_protected.setter
+    def mgm_key_protected(self, value):
+        self._set_flag(0x02, value)
+
     def get_bytes(self):
         data = b''
         if self._flags is not None:
@@ -376,6 +384,19 @@ class PivmanData(object):
         return Tlv(0x80, data)
 
 
+class PivmanProtectedData(object):
+
+    def __init__(self, raw_data=Tlv(0x88)):
+        data = _parse_tlv_dict(Tlv(raw_data).value)
+        self.key = data.get(0x89)
+
+    def get_bytes(self):
+        data = b''
+        if self.key is not None:
+            data += Tlv(0x89, self.key)
+        return Tlv(0x88, data)
+
+
 class PivController(object):
 
     def __init__(self, driver):
@@ -392,10 +413,18 @@ class PivController(object):
     def version(self):
         return self._version
 
+    @property
+    def has_protected_key(self):
+        return self.has_derived_key or self.has_stored_key
+
     @property
     def has_derived_key(self):
         return self._pivman_data.salt is not None
 
+    @property
+    def has_stored_key(self):
+        return self._pivman_data.mgm_key_protected
+
     @property
     def puk_blocked(self):
         return self._pivman_data.puk_blocked
@@ -421,6 +450,17 @@ class PivController(object):
     def _read_version(self):
         return tuple(six.iterbytes(self.send_cmd(INS.GET_VERSION)))
 
+    def _init_pivman_protected(self):
+        try:
+            self._pivman_protected_data = PivmanProtectedData(
+                self.get_data(OBJ.PIVMAN_PROTECTED_DATA))
+        except APDUError as e:
+            if e.sw == SW_APPLICATION_NOT_FOUND:
+                # No data there, initialise a new object.
+                self._pivman_protected_data = PivmanProtectedData()
+            else:
+                raise
+
     def verify(self, pin, touch_callback=None):
         try:
             self.send_cmd(INS.VERIFY, 0, PIN, _pack_pin(pin))
@@ -428,10 +468,15 @@ class PivController(object):
             raise ValueError(
                 'Pin verification failed. {} tries left.'.format(
                         self.get_pin_tries()))
+
         if self.has_derived_key and not self._authenticated:
             self.authenticate(
                 _derive_key(pin, self._pivman_data.salt), touch_callback)
 
+        if self.has_stored_key and not self._authenticated:
+            self._init_pivman_protected()
+            self.authenticate(self._pivman_protected_data.key, touch_callback)
+
     def change_pin(self, old_pin, new_pin):
         self.send_cmd(INS.CHANGE_REFERENCE, 0, PIN,
                       _pack_pin(old_pin) + _pack_pin(new_pin))
@@ -488,12 +533,14 @@ class PivController(object):
             touch_timer = Timer(0.500, touch_callback)
             touch_timer.start()
 
-        pt2 = self.send_cmd(INS.AUTHENTICATE, ALGO.TDES, SLOT.CARD_MANAGEMENT,
-                            Tlv(TAG.DYN_AUTH, Tlv(0x80, pt1) + Tlv(0x81, ct2))
-                            )[4:12]
-
-        if touch_callback is not None:
-            touch_timer.cancel()
+        try:
+            pt2 = self.send_cmd(
+                INS.AUTHENTICATE, ALGO.TDES, SLOT.CARD_MANAGEMENT,
+                Tlv(TAG.DYN_AUTH, Tlv(0x80, pt1) + Tlv(0x81, ct2))
+                )[4:12]
+        finally:
+            if touch_callback is not None:
+                touch_timer.cancel()
 
         encryptor = cipher.encryptor()
         pt2_cmp = encryptor.update(ct2) + encryptor.finalize()
@@ -501,13 +548,37 @@ class PivController(object):
             raise ValueError('Device challenge did not match!')
         self._authenticated = True
 
-    def set_mgm_key(self, new_key, touch=False):
+    def set_mgm_key(self, new_key, touch=False, store_on_device=False):
+        # Set the new management key
         self.send_cmd(
             INS.SET_MGMKEY, 0xff, 0xfe if touch else 0xff,
             six.int2byte(ALGO.TDES) + Tlv(SLOT.CARD_MANAGEMENT, new_key))
         if self.has_derived_key:
+            # Clear salt for old derived keys.
             self._pivman_data.salt = None
-            self.put_data(OBJ.PIVMAN_DATA, self._pivman_data.get_bytes())
+        # Set flag for stored or not stored key.
+        self._pivman_data.mgm_key_protected = store_on_device
+        # Update readable pivman data
+        self.put_data(OBJ.PIVMAN_DATA, self._pivman_data.get_bytes())
+        if store_on_device:
+            # Store key in protected pivman data
+            self._init_pivman_protected()
+            self._pivman_protected_data.key = new_key
+            self.put_data(
+                OBJ.PIVMAN_PROTECTED_DATA,
+                self._pivman_protected_data.get_bytes())
+        elif not store_on_device and self.has_stored_key:
+            # If new key should not be stored and there is an old stored key,
+            # try to clear it.
+            try:
+                self._init_pivman_protected()
+                self._pivman_protected_data.key = None
+                self.put_data(
+                    OBJ.PIVMAN_PROTECTED_DATA,
+                    self._pivman_protected_data.get_bytes())
+            except APDUError:
+                # No pin provided, can't clear key..
+                pass
 
     def get_pin_tries(self):
         """
