@@ -281,8 +281,8 @@ def hmac_shorten_key(key, algo):
     return key
 
 
-def time_challenge(timestamp):
-    return struct.pack('>q', int(timestamp // 30))
+def time_challenge(timestamp, period=30):
+    return struct.pack('>q', int(timestamp // period))
 
 
 def parse_uri(val):
@@ -293,11 +293,6 @@ def parse_uri(val):
         params = dict((k, v[0]) for k, v in parse_qs(parsed.query).items())
         params['name'] = unquote(parsed.path)[1:]  # Unquote and strip leading /
         params['type'] = parsed.hostname
-        # Issuer can come both in a param and inside name param.
-        # We store both in the name field on the key.
-        if 'issuer' in params \
-                and not params['name'].startswith(params['issuer']):
-                    params['name'] = params['issuer'] + ':' + params['name']
         return params
     except:
         raise ValueError('URI seems to have the wrong format.')
